package valinta;

/**
 * Luokka äänestyksen vaihtoehdoille
 * @author Vesa Lappalainen
 * @version 1.0, 27.02.2003
 * @version 1.1, 06.02.2011, vesal, html
 */
public class Vaihtoehdot2 {
    private Valinta alkiot[];
    private int lkm = 0;


    /**
     * Alustetaan vaihtoehtojen lukumäärä
     * @param maxKoko maksimimäärä vaihtoehtoja
     */
    public Vaihtoehdot2(int maxKoko) {
        alkiot = new Valinta[maxKoko];
    }


    /**
     * Lisää uusi valinta.
     * @param uusi lisättävä valinta
     * 
     * @example
     * <pre name="test">
     * Vaihtoehdot2 vaihtoehdot = new Vaihtoehdot2(3);
     * vaihtoehdot.lisaa(new Valinta('K',"Kyllä"));  vaihtoehdot.getLkm() === 1;  // NOPMD
     * vaihtoehdot.lisaa(new Valinta('E',"Ei"));     vaihtoehdot.getLkm() === 2;
     * vaihtoehdot.lisaa(new Valinta('H',"Ehkä"));   vaihtoehdot.getLkm() === 3;  // NOPMD
     * </pre>
     */
    public void lisaa(Valinta uusi) {
        if ( lkm >= alkiot.length ) return; // Jotakin hienompaa
        alkiot[lkm] = uusi;
        lkm++;
    }


    /**
     * Palautetaan äänestyskohteiden lukumäärä
     * @return äänestyskohteiden lukumäärä 
     */
    public int getLkm() {
        return lkm;
    }


    /**
     * Palauttaa viitteen i:nteen alkioon
     * @param i monenko alkion viite palautetaan
     * @return viite valittuun alkioon
     * @throws IndexOutOfBoundsException jos indeksi väärissä rajoissa
     * 
     * @example
     * <pre name="test">
     * Vaihtoehdot2 vaihtoehdot = new Vaihtoehdot2(3);
     * vaihtoehdot.lisaa(new Valinta('K',"Kyllä"));     // NOPMD
     * vaihtoehdot.lisaa(new Valinta('E',"Ei"));  
     * vaihtoehdot.lisaa(new Valinta('H',"Ehkä"));
     * vaihtoehdot.anna(0).getTulostus() === "K = Kyllä"; 
     * vaihtoehdot.anna(1).getTulostus() === "E = Ei"; 
     * vaihtoehdot.anna(2).getTulostus() === "H = Ehkä"; 
     * vaihtoehdot.anna(3).getTulostus() === "H = Ehkä";  #THROWS IndexOutOfBoundsException 
     * </pre>
     */
    public Valinta anna(int i) throws IndexOutOfBoundsException {
        return alkiot[i];
    }


    /**
     * Etsitään valittua kohdetta kirjaimen perusteella
     * @param painettu etsittävän kohteen kirjain
     * @return löydetty kohde tai null jos ei löydy
     * 
     * @example
     * <pre name="test">
     * Vaihtoehdot2 vaihtoehdot = new Vaihtoehdot2(3);
     * vaihtoehdot.lisaa(new Valinta('K',"Kyllä"));
     * vaihtoehdot.lisaa(new Valinta('E',"Ei"));
     * vaihtoehdot.lisaa(new Valinta('H',"Ehkä"));
     * vaihtoehdot.etsi('K').getTeksti() === "Kyllä";
     * vaihtoehdot.etsi('E').getTeksti() === "Ei";
     * vaihtoehdot.etsi('H').getTeksti() === "Ehkä";
     * vaihtoehdot.etsi('k') === null;
     * </pre>
     */
    public Valinta etsi(char painettu) {
        for (int i = 0; i < lkm; i++)
            if ( alkiot[i].getKirjain() == painettu )
                return alkiot[i];
        return null;
    }


    /**
     * Tulostetaan äänestyksen tämän hetkinen tilanne
     * @return tulokset html-taulukkona
     * @example
     * <pre name="test">
     *   Vaihtoehdot2 valinnat = new Vaihtoehdot2(10);
     *   valinnat.lisaa(new Valinta('T',"Tyttö"));
     *   valinnat.lisaa(new Valinta('P',"Poika"));
     *
     *   String table = valinnat.aanetAsHTMLTable();
     *   String tulos = "[table]" +
     *     "[tr][th]Tilanne:[/th][/tr]" +
     *     "[tr][td]Tyttö[/td][td]0[/td][/tr]" +
     *     "[tr][td]Poika[/td][td]0[/td][/tr]" +
     *     "[/table]";
     *   tulos = tulos.replaceAll("\\[","<").replaceAll("\\]",">");  
     *   table === tulos;
     * </pre>       
     */
    public String aanetAsHTMLTable() {
        StringBuilder table = new StringBuilder("<table>");
        table.append("<tr><th>Tilanne:</th></tr>");
        for (int i = 0; i < getLkm(); i++) {
            Valinta valinta = anna(i);
            table.append("<tr><td>"+valinta.getTeksti()+"</td><td>" + valinta.getMaara()+"</td></tr>");
        }
        table.append("</table>");
        return table.toString();
    }

    
    /**
     * Palauttaa äänestyksen kohteet merkkijonotaulukkona
     * @return kohteet merkkijonotaulukkona.
     * @example
     * <pre name="test">
     *   Vaihtoehdot2 valinnat = new Vaihtoehdot2(10);
     *   valinnat.lisaa(new Valinta('T',"Tyttö"));
     *   valinnat.lisaa(new Valinta('P',"Poika"));
     *
     *   String[] kohteet = valinnat.getKohteet();
     *   kohteet.length === 2;
     *   kohteet[0] === "Tyttö";
     *   kohteet[1] === "Poika";
     * </pre>       
     */
    public String[] getKohteet() {
       String[] kohteet = new String[getLkm()];
       for (int i = 0; i < getLkm(); i++) {
           Valinta valinta = anna(i);
           kohteet[i] = valinta.getTeksti();
       }    
       return kohteet;
    }
    
    
    /**
     * Palauttaa äänestyksen valintakirjaimet kohteita vastaavassa järjestyksessä.
     * @return valintakirjaimet
     * @example
     * <pre name="test">
     *   Vaihtoehdot2 valinnat = new Vaihtoehdot2(10);
     *   valinnat.lisaa(new Valinta('T',"Tyttö"));
     *   valinnat.lisaa(new Valinta('P',"Poika"));
     *
     *   valinnat.getKirjaimet() === "TP";
     * </pre>       
     */
    public String getKirjaimet() {
       StringBuilder kirjaimet = new StringBuilder();
       for (int i = 0; i < getLkm(); i++) {
           Valinta valinta = anna(i);
           kirjaimet.append(valinta.getKirjain());
       }    
       return kirjaimet.toString();
    }
    
}